//////////////////////////////////////////////////////////
//	Microchip C18 Tutorial-Motor Velocity Controller	//
//	by Michael Hwang									//
//	Northwestern University								//
//	Summer 2007											//
//////////////////////////////////////////////////////////
/*
This program implements a closed-loop feedback velocity controller for
a motor with a quadrature encoder on its shaft.  Use HyperTerminal on a PC
to control the motor.  In HyperTerminal, select a baud rate of 115200 bps
and use ANSI emulation.
*/
#include <p18f4431.h>
#include <usart.h>
#include <stdlib.h>
#include <stdio.h>

#include "main.h"
#include "motor_pwm.h"
#include "QEI.h"
#include "Lead_Lag(tutorial).h"

//	Set up configuration bits:
//	using Internal Oscillator, and multiplexing PWM4 on RB5
#pragma config OSC=IRCIO, PWM4MX = RB5  

void ISR(void);
	//Prototype for Interrupt Service Routine
	
#pragma code low_vector_section=0x18	//place code at the low-priority interrupt vector (0x18)
void interrupt_at_low_vector(void){
	_asm GOTO ISR _endasm
		//since the interrupt vector can only hold 1 instruction,
		//we use the assembly GOTO instruction to make it run the ISR.		
}
#pragma code

#pragma interrupt ISR
void ISR(void){
	int speedPercent;
	extern int targetSpeed;
	char strTargetSpeedBuf[20];
		//ISRs can't access global variables directly, we use the 'extern' keyword
		//to indicate that the variable is defined elswhere.
		
	PORTBbits.RB5=1; 	// RB5 is turned on each time this ISR runs as an indicator
	
	sprintf(strTargetSpeedBuf,"%i    \r\n\t", targetSpeed);	//print targetSpeed
	
	while (BusyUSART());
	putcUSART('\t');	//horizontal tab
	putsUSART(strTargetSpeedBuf);
	
	speedPercent = getLeadLag(targetSpeed);
	setPDC0(speedPercent);
	
	MOVE_CURSOR_UP  		//defined in "main.h"
	
	PORTBbits.RB5=0;
	PIR3bits.TMR5IF = 0;	//reset interrupt flag 
}


///////////////Global Variables////////////////
volatile int targetSpeed = 0;	

//main function
void main(void){
	char inputBuffer; //buffer to hold input commands;
	OSCCON=0b11111111; //set oscillator for 8MHz
	
		//Initialize data ports and latches.  Set unused ports to output low 
		//to avoid floating inputs.
	TRISA=0;
	ANSEL0=0;
	ANSEL1=0;
	TRISB=0;
	TRISC=0b10000000; //RC7 is the USART receiver pin
	TRISD=0;
	TRISE=0;
	LATA=0;
	LATB=0;
	LATC=0;
	LATD=0;
	LATE=0;

	

	BAUDCTL = 0b00001000; //16bit baud generator, no wake-up, no auto-detect
		//Beware that sending data to the serial port takes a lot of time.  If your
		//baudrate is too low, it may not be able to finish executing in time.		
	OpenUSART(USART_TX_INT_OFF &
				USART_RX_INT_OFF &
				USART_ASYNCH_MODE &
				USART_EIGHT_BIT &
				USART_CONT_RX &
				USART_BRGH_HIGH,16); //115200 baud.
				
				
		//set the duty cycle of PWM module 0 (PWM0,PWM1)
		// so that the motor will be stopped (duty cycle at 50%).
	setPDC0(0);	
	
		//turn on PWM0 and PWM1, and set them to be complementary (inverses of each other)
	enablePWM(	PWM_EN_0_1 &
				PWM_01_COMPLEMENTARY);
				
	configPWMFreq(18000);	//configure PWM frequency
	
		//initialize the QEI to use 4x mode, 
		//this function also sets RA3 and RA4 as digital inputs.
	initQEI(4);		
					
		//print welcome message
	printf("\n\n\n\r");
	printf("\
**************************************\r\n\
* Lead-Lag Motor Velocity Controller *\r\n\
* Press 'a' to accelerate forwards   *\r\n\
* Press 'd' to accererate backwards  *\r\n\
* Press 's' to stop                  *\r\n\
**************************************\r\n");
printf("Speed (encoder counts per period):\r\n");
printf("Target:\r\nActual:\r");

MOVE_CURSOR_UP

//This macro calculates the match register value for the Timer5 interrupt
//The period "T" is defined in file "lead_lag(tutorial).h"
#define PERIOD_COUNT(period,prescale) ((FOSC/4)*period/prescale-1) 
	T5CON = 0b00011000; //1:8 prescale
	PR5H = (unsigned int)PERIOD_COUNT(T,8)/256;
	PR5L = (unsigned int)PERIOD_COUNT(T,8)%256; 
	
		//Set up interrupt:
	RCONbits.IPEN=0;      //Interrupt Prioritization disabled;
	PIE3bits.TMR5IE = 1;  //Enable Timer5 Interrupt
	INTCONbits.GIE = 1;   //Global Interrupt Enable
	INTCONbits.PEIE = 1;  //Enable Peripheral Interrupts
	T5CONbits.TMR5ON = 1; //Turn on Timer5 
	
	while(1){				
			//If buffer overrun error occurred, 
			//disable then enable the receiver to clear the error
		if(RCSTAbits.OERR==1){
			RCSTAbits.CREN=0;
			RCSTAbits.CREN=1;
		}
			//If framing error occurred, clear the buffer
		if(RCSTAbits.FERR==1){
			ReadUSART();
		}
		if(DataRdyUSART()){
			inputBuffer=ReadUSART();
		}
		switch (inputBuffer){
			case 's':
			targetSpeed=0;	//set target speed to zero
			break;
			
			case 'a':
			targetSpeed+=1;	//increase target speed
			break;
			
			case 'd':
			targetSpeed-=1;	//decrease target speed
			break;
		}
	inputBuffer=0; //clear buffer

	}	//end while(1)
}	//end main()

